#!/usr/bin/env bash

# This script upgrades all Python packages in `venv`.
# The basic steps are:
# *   remove all package version records in `@/conf/dev_env_packages.txt`
# *   uninstall everything from `venv`
# *   let `@/exe/bootstrap_dev_env.bash` re-install all (transitive) dependencies at their latest versions

# Define with `s` in value to debug:
if [[ "${ARGRELAY_DEBUG-}" == *s* ]]
then
    set -x
    set -v
fi

# Debug: Print commands before execution:
#set -x
# Debug: Print commands after reading from a script:
#set -v
# Return non-zero exit code from commands within a pipeline:
set -o pipefail
# Exit on non-zero exit code from a command:
set -e
# Inherit trap on ERR by sub-shells:
set -E
# Error on undefined variables:
set -u

script_source="${BASH_SOURCE[0]}"
# shellcheck disable=SC2034
script_name="$( basename -- "${script_source}" )"
# Absolute script dirname:
# https://stackoverflow.com/a/246128/441652
script_dir="$( cd -- "$( dirname -- "${script_source}" )" &> /dev/null && pwd )"
# FS_29_54_67_86 dir_structure: `@/exe/` -> `@/`:
argrelay_dir="$( dirname "${script_dir}" )"

# TODO_64_79_28_85: deduplicate this file with publish_package.bash (move upgrade function here from there).

# Switch to `@/` to avoid creating temporary dirs somewhere else:
cd "${argrelay_dir}" || exit 1

# Ensure the script was started in `@/exe/dev_shell.bash`:
if [[ -z "${ARGRELAY_DEV_SHELL:-}" ]]
then
    echo "ERROR: Run this script under \`@/exe/dev_shell.bash\`." 1>&2
    exit 1
fi

# Ensure it is a `venv` (`@/exe/dev_shell.bash` activates `venv` configured in `@/conf/python_conf.bash`):
test -n "${VIRTUAL_ENV}"

# Clear package versions:
true > "conf/dev_env_packages.txt"

# Clear `venv` (only to be restored in the next step):
pip uninstall -y -r <( pip freeze )

# Bootstrap to let `@/exe/deploy_project.bash` install all packages
# via transitive dependencies and at their latest versions:
"${argrelay_dir}/exe/bootstrap_dev_env.bash"
